@@ -350,18 +350,16 @@ DecodeHeader (guchar *Data, gint Bytes, struct ico_progressive_state *State)
 
 	g_assert(State->LineBuf != NULL);
 
-
 	if (State->pixbuf == NULL) {
-		if (State->Type == 32)
-			State->pixbuf =
-			    gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
-					   (gint) State->Header.width,
-					   (gint) State->Header.height);
-		else
-			State->pixbuf =
-			    gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
-					   (gint) State->Header.width,
-					   (gint) State->Header.height);
+		/* ICOs always have an alpha channel, which will be set in a
+		 * second pass after the main pixel data has been read.
+		 */
+		State->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
+						(gint) State->Header.width,
+						(gint) State->Header.height);
+		/* FIXME: gdk_pixbuf_new() could return NULL and we are not
+		 * dealing with it!
+		 */
 
 		if (State->prepared_func != NULL)
 			/* Notify the client that we are ready to go */
